@@ -153,6 +153,7 @@ alloc_error:
 
 // the static analyzer can't prove the allocs to be correct
 #pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wunknown-warning-option"
 #pragma GCC diagnostic ignored "-Wanalyzer-malloc-leak"
 
 static data_t *vdata_make(data_t *first, const char *key, const char *pretty_key, va_list ap)
@@ -180,10 +181,13 @@ static data_t *vdata_make(data_t *first, const char *key, const char *pretty_key
                 fprintf(stderr, "vdata_make() format type used twice\n");
                 goto alloc_error;
             }
-            format = strdup(va_arg(ap, char *));
-            if (!format) {
-                WARN_STRDUP("vdata_make()");
-                goto alloc_error;
+            format = va_arg(ap, char *);
+            if (format) {
+                format = strdup(format);
+                if (!format) {
+                    WARN_STRDUP("vdata_make()");
+                    goto alloc_error;
+                }
             }
             type = va_arg(ap, data_type_t);
             continue;
@@ -202,7 +206,7 @@ static data_t *vdata_make(data_t *first, const char *key, const char *pretty_key
             break;
         case DATA_STRING:
             value_release = (value_release_fn)free; // appease CSA checker
-            value.v_ptr = strdup(va_arg(ap, char *));
+            value.v_ptr = strdup(va_arg(ap, char const *));
             if (!value.v_ptr)
                 WARN_STRDUP("vdata_make()");
             break;
@@ -283,7 +287,7 @@ R_API data_t *data_make(const char *key, const char *pretty_key, ...)
     return result;
 }
 
-R_API data_t *data_append(data_t *first, const char *key, const char *pretty_key, ...)
+static data_t *data_append(data_t *first, const char *key, const char *pretty_key, ...)
 {
     va_list ap;
     va_start(ap, pretty_key);
@@ -310,6 +314,42 @@ R_API data_t *data_prepend(data_t *first, const char *key, const char *pretty_ke
     return result;
 }
 
+// Wrappers for now, should be refactored.
+R_API data_t *data_int(data_t *first, char const *key, char const *pretty_key, char const *format, int val)
+{
+    return data_append(first, key, pretty_key, DATA_FORMAT, format, DATA_INT, val, NULL);
+}
+R_API data_t *data_dbl(data_t *first, char const *key, char const *pretty_key, char const *format, double val)
+{
+    return data_append(first, key, pretty_key, DATA_FORMAT, format, DATA_DOUBLE, val, NULL);
+}
+R_API data_t *data_str(data_t *first, char const *key, char const *pretty_key, char const *format, char const *val)
+{
+    return data_append(first, key, pretty_key, DATA_FORMAT, format, DATA_STRING, val, NULL);
+}
+R_API data_t *data_ary(data_t *first, char const *key, char const *pretty_key, char const *format, data_array_t *val)
+{
+    return data_append(first, key, pretty_key, DATA_FORMAT, format, DATA_ARRAY, val, NULL);
+}
+R_API data_t *data_dat(data_t *first, char const *key, char const *pretty_key, char const *format, data_t *val)
+{
+    return data_append(first, key, pretty_key, DATA_FORMAT, format, DATA_DATA, val, NULL);
+}
+R_API data_t *data_hex(data_t *first, char const *key, char const *pretty_key, char const *format, uint8_t const *val, unsigned len, char *buf)
+{
+    if (!format || !*format) {
+        format = "%02x";
+    }
+
+    char *p = buf;
+    for (unsigned i = 0; i < len; i += 1) {
+        p += sprintf(p, format, val[i]);
+    }
+    *p = '\0';
+
+    return data_append(first, key, pretty_key, DATA_FORMAT, NULL, DATA_STRING, buf, NULL);
+}
+
 R_API void data_array_free(data_array_t *array)
 {
     array_element_release_fn release = dmt[array->type].array_element_release;
@@ -329,6 +369,10 @@ R_API data_t *data_retain(data_t *data)
     return data;
 }
 
+#if defined(__clang__)
+    // ignore "call to function _free through pointer to incorrect function type"
+    __attribute__((no_sanitize("undefined")))
+#endif
 R_API void data_free(data_t *data)
 {
     if (data && data->retain) {
