@@ -74,7 +74,7 @@ struct flex_get {
     const char *format;
 };
 
-#define GETTER_SLOTS 8
+#define GETTER_SLOTS 12
 
 struct flex_params {
     char *name;
@@ -125,22 +125,12 @@ static void render_getters(data_t *data, uint8_t *bits, struct flex_params *para
         int m;
         for (m = 0; getter->map[m].val; m++) {
             if (getter->map[m].key == val) {
-                data_append(data,
-                        getter->name, "", DATA_STRING, getter->map[m].val,
-                        NULL);
+                data_str(data, getter->name, "", NULL, getter->map[m].val);
                 break;
             }
         }
         if (!getter->map[m].val) {
-            if (getter->format) {
-                data_append(data,
-                    getter->name, "", DATA_FORMAT, getter->format, DATA_INT, val,
-                    NULL);
-            } else {
-                data_append(data,
-                        getter->name, "", DATA_INT, val,
-                        NULL);
-            }
+            data_int(data, getter->name, "", getter->format, val);
         }
     }
 }
@@ -157,7 +147,7 @@ static int flex_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     char *row_codes[BITBUF_ROWS];
     char row_bytes[BITBUF_ROWS * BITBUF_COLS * 2 + 1]; // TODO: this is a lot of stack
 
-    struct flex_params *params = decoder->decode_ctx;
+    struct flex_params *params = decoder_user_data(decoder);
 
     // discard short / unwanted bitbuffers
     if ((bitbuffer->num_rows < params->min_rows)
@@ -267,9 +257,7 @@ static int flex_callback(r_device *decoder, bitbuffer_t *bitbuffer)
         }
     }
 
-    if (decoder->verbose) {
-        decoder_log_bitbuffer(decoder, 1, params->name, bitbuffer, "");
-    }
+    decoder_log_bitbuffer(decoder, 1, params->name, bitbuffer, "");
 
     // discard duplicates
     if (params->unique) {
@@ -426,6 +414,56 @@ static void help(void)
     exit(0);
 }
 
+static float parse_atoiv(char const *str, int def, char const *error_hint)
+{
+    if (!str) {
+        return def;
+    }
+
+    if (!*str) {
+        return def;
+    }
+
+    char *endptr;
+    int val = strtol(str, &endptr, 10);
+
+    if (str == endptr) {
+        fprintf(stderr, "%sinvalid number argument (%s)\n", error_hint, str);
+        exit(1);
+    }
+
+    return val;
+}
+
+static float parse_float(char const *str, char const *error_hint)
+{
+    if (!str) {
+        fprintf(stderr, "%smissing number argument\n", error_hint);
+        exit(1);
+    }
+
+    if (!*str) {
+        fprintf(stderr, "%sempty number argument\n", error_hint);
+        exit(1);
+    }
+
+    char *endptr;
+    double val = strtod(str, &endptr);
+
+    if (str == endptr) {
+        fprintf(stderr, "%sinvalid number argument (%s)\n", error_hint, str);
+        exit(1);
+    }
+
+    if (*endptr != '\0') {
+        fprintf(stderr, "%strailing characters in number argument (%s)\n", error_hint, str);
+        exit(1);
+    }
+
+    return val;
+
+}
+
 static unsigned parse_modulation(char const *str)
 {
     if (!strcasecmp(str, "OOK_MC_ZEROBIT"))
@@ -571,7 +609,6 @@ static void parse_getter(const char *arg, struct flex_get *getter)
         usage();
     }
     /*
-    if (decoder->verbose)
         fprintf(stderr, "parse_getter() bit_offset: %d bit_count: %d mask: %lx name: %s\n",
                 getter->bit_offset, getter->bit_count, getter->mask, getter->name);
     */
@@ -586,18 +623,11 @@ r_device *flex_create_device(char *spec)
         help();
     }
 
-    struct flex_params *params = calloc(1, sizeof(*params));
-    if (!params) {
-        WARN_CALLOC("flex_create_device()");
-        return NULL; // NOTE: returns NULL on alloc failure.
-    }
-    r_device *dev = calloc(1, sizeof(*dev));
+    r_device *dev = decoder_create(NULL, sizeof(struct flex_params));
     if (!dev) {
-        WARN_CALLOC("flex_create_device()");
-        free(params);
         return NULL; // NOTE: returns NULL on alloc failure.
     }
-    dev->decode_ctx = params;
+    struct flex_params *params = decoder_user_data(dev);
     int get_count = 0;
 
     spec = strdup(spec);
@@ -629,45 +659,45 @@ r_device *flex_create_device(char *spec)
         else if (!strcasecmp(key, "m") || !strcasecmp(key, "modulation"))
             dev->modulation = parse_modulation(val);
         else if (!strcasecmp(key, "s") || !strcasecmp(key, "short"))
-            dev->short_width = atoi(val);
+            dev->short_width = parse_float(val, "short: ");
         else if (!strcasecmp(key, "l") || !strcasecmp(key, "long"))
-            dev->long_width = atoi(val);
+            dev->long_width = parse_float(val, "long: ");
         else if (!strcasecmp(key, "y") || !strcasecmp(key, "sync"))
-            dev->sync_width = atoi(val);
+            dev->sync_width = parse_float(val, "sync: ");
         else if (!strcasecmp(key, "g") || !strcasecmp(key, "gap"))
-            dev->gap_limit = atoi(val);
+            dev->gap_limit = parse_float(val, "gap: ");
         else if (!strcasecmp(key, "r") || !strcasecmp(key, "reset"))
-            dev->reset_limit = atoi(val);
+            dev->reset_limit = parse_float(val, "reset: ");
         else if (!strcasecmp(key, "t") || !strcasecmp(key, "tolerance"))
-            dev->tolerance = atoi(val);
+            dev->tolerance = parse_float(val, "tolerance: ");
         else if (!strcasecmp(key, "prio") || !strcasecmp(key, "priority"))
-            dev->priority = atoi(val);
+            dev->priority = parse_atoiv(val, 0, "priority: ");
 
         else if (!strcasecmp(key, "bits>"))
-            params->min_bits = val ? atoi(val) : 0;
+            params->min_bits = parse_atoiv(val, 0, "bits: ");
         else if (!strcasecmp(key, "bits<"))
-            params->max_bits = val ? atoi(val) : 0;
+            params->max_bits = parse_atoiv(val, 0, "bits: ");
         else if (!strcasecmp(key, "bits"))
-            params->min_bits = params->max_bits = val ? atoi(val) : 0;
+            params->min_bits = params->max_bits = parse_atoiv(val, 0, "bits:");
 
         else if (!strcasecmp(key, "rows>"))
-            params->min_rows = val ? atoi(val) : 0;
+            params->min_rows = parse_atoiv(val, 0, "rows: ");
         else if (!strcasecmp(key, "rows<"))
-            params->max_rows = val ? atoi(val) : 0;
+            params->max_rows = parse_atoiv(val, 0, "rows: ");
         else if (!strcasecmp(key, "rows"))
-            params->min_rows = params->max_rows = val ? atoi(val) : 0;
+            params->min_rows = params->max_rows = parse_atoiv(val, 0, "rows: ");
 
         else if (!strcasecmp(key, "repeats>"))
-            params->min_repeats = val ? atoi(val) : 0;
+            params->min_repeats = parse_atoiv(val, 0, "repeats: ");
         else if (!strcasecmp(key, "repeats<"))
-            params->max_repeats = val ? atoi(val) : 0;
+            params->max_repeats = parse_atoiv(val, 0, "repeats: ");
         else if (!strcasecmp(key, "repeats"))
-            params->min_repeats = params->max_repeats = val ? atoi(val) : 0;
+            params->min_repeats = params->max_repeats = parse_atoiv(val, 0, "repeats: ");
 
         else if (!strcasecmp(key, "invert"))
-            params->invert = val ? atoi(val) : 1;
+            params->invert = parse_atoiv(val, 1, "invert: ");
         else if (!strcasecmp(key, "reflect"))
-            params->reflect = val ? atoi(val) : 1;
+            params->reflect = parse_atoiv(val, 1, "reflect: ");
 
         else if (!strcasecmp(key, "match"))
             params->match_len = parse_bits(val, params->match_bits);
@@ -676,15 +706,15 @@ r_device *flex_create_device(char *spec)
             params->preamble_len = parse_bits(val, params->preamble_bits);
 
         else if (!strcasecmp(key, "countonly"))
-            params->count_only = val ? atoi(val) : 1;
+            params->count_only = parse_atoiv(val, 1, "countonly: ");
 
         else if (!strcasecmp(key, "unique"))
-            params->unique = val ? atoi(val) : 1;
+            params->unique = parse_atoiv(val, 1, "unique: ");
 
         else if (!strcasecmp(key, "decode_uart"))
-            params->decode_uart = val ? atoi(val) : 1;
+            params->decode_uart = parse_atoiv(val, 1, "decode_uart: ");
         else if (!strcasecmp(key, "decode_dm"))
-            params->decode_dm = val ? atoi(val) : 1;
+            params->decode_dm = parse_atoiv(val, 1, "decode_dm: ");
 
         else if (!strcasecmp(key, "symbol_zero"))
             params->symbol_zero = parse_symbol(val);
@@ -776,13 +806,11 @@ r_device *flex_create_device(char *spec)
     }
 
     /*
-    if (decoder->verbose) {
         fprintf(stderr, "Adding flex decoder \"%s\"\n", params->name);
         fprintf(stderr, "\tmodulation=%u, short_width=%.0f, long_width=%.0f, reset_limit=%.0f\n",
                 dev->modulation, dev->short_width, dev->long_width, dev->reset_limit);
         fprintf(stderr, "\tmin_rows=%u, min_bits=%u, min_repeats=%u, invert=%u, reflect=%u, match_len=%u, preamble_len=%u\n",
                 params->min_rows, params->min_bits, params->min_repeats, params->invert, params->reflect, params->match_len, params->preamble_len);
-    }
     */
 
     free(spec);
