@@ -80,7 +80,7 @@ pulse_count/(n_imp_per_kwh)
 
 #include "decoder.h"
 
-static uint16_t checksum_taps[] = {
+static uint16_t const checksum_taps[] = {
         0x4880, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
         0x2080, 0x4000, 0x4000, 0x4000, 0x4000, 0x4000, 0x4000,
 };
